#! /usr/bin/perl -w

package MultiTask;

use Exporter;
@ISA=('Exporter');
@EXPORT=('MultiTaskGo', 'mtSetSgeDir', 'mtSetSgeName', 'mtSetSgeScriptName', 'mtSetSgeArgs');

$runner="/bin/bash";

############################################################
# exported sub's

my $sgeArgs = "";
my $sgeQueues = "-q all.q\@\@stable";
my $sgeName;
my $sgeDir;
my $sgeRunner;
my $sgeScriptName;

sub MakeSgeCommand($$@)
{
	$indexFrom = shift;
	$indexTo = shift;
	my @tasks = @_;
	if (!defined($sgeDir)) {
		die "\$sgeDir must be defined for submitting - use mtSetSgeDir(\"/path/to/experiment/root\")";
	}
	if (!defined($sgeName)) {
		die "\$sgeName must be defined for submitting - use mtSetSgeName(\"TheName\")";
	}
	if (!defined($sgeScriptName)) {
		die "\$sgeScriptName must be defined for submitting - use mtSetSgeScriptName(\"script.pl\")";
	}
	return "qsub -N $sgeName $sgeArgs $sgeQueues -t $indexFrom-$indexTo -o $sgeDir/sge.out -e $sgeDir/sge.err $sgeScriptName sgeX";
}
sub MultiTaskGo
{
	my $mode = shift;
	if (!defined($mode)) {
		warn "no mode specified - will use list\n";
		$mode = "list";
	}
	my  @tasks = @_;
	if ($mode eq "nothing") {
		warn "MultiTask is doing NOTHING";
	} elsif ($mode eq "count") {
		$count = $#tasks+1;
		print "total tasks: $count   (max range: 1-$count)\n";
	} elsif ($mode eq "list") {
		# list
		for ($i = 1; $i <= $#tasks+1; $i++) {
			$task = $tasks[$i-1];
			print "$i: $task\n";
		}
	} elsif ($mode =~ /^list:(\d+)-(\d+)$/) {
		# list:NN-NN
		for ($i = $1; $i <= $2; $i++) {
			$task = $tasks[$i-1];
			print "$i: $task\n";
		}
	} elsif ($mode =~ /^run:(\d+)-(\d+)$/) {
		# run:NN-NN
		for ($i = $1; $i <= $2; $i++) {
			$task = $tasks[$i-1];
			print "MultiTask running ($i): $runner -c '$task'\n";
			system $runner, "-c", "$task";
		}
	} elsif ($mode =~ /^sgepreview:(\d+)-(\d+)$/) {
		# sgepreview
		my $cmd = MakeSgeCommand($1, $2, @tasks);
		print "$cmd\n";
	} elsif ($mode =~ /^sgesubmit:(\d+)-(\d+)$/) {
		my $cmd = MakeSgeCommand($1, $2, @tasks);
		print "MultiTask running: $cmd\n";
		system $runner, "-c", "$cmd";
	} elsif ($mode eq "sgeX") {
		$taskId = $ENV{SGE_TASK_ID};
		$task = $tasks[$taskId-1];
		print "MultiTask\@SGE running task id: $taskId -- $task\n";
		system $runner, "-c", "$task";
	} else {
		die "Unrecognized commandline option: $mode\n";
	}
}

sub mtSetSgeDir($) {
	$sgeDir = shift;
}
sub mtSetSgeName($) {
	$sgeName = shift;
}
sub mtSetSgeScriptName($) {
	$sgeScriptName = shift;
}
sub mtSetSgeArgs($) {
	$sgeArgs = shift;
}


############################################################
# some init?
1;
