#! /usr/bin/perl -w

use Cwd 'abs_path';

my @tasks=();		#there all tasks will be stored;
my @shrinktasks=();	#all shrink tasks will be stored here
my @order=(2**-11,2**-10,2**-9,2**-8,2**-7,2**-6,2**-5,2**-4,2**-3,2**-2,2**-1,2**0,
	    2**1,2**2,2**3,2**4,2**5,2**6,2**7,2**8,2**9,2**10,2**11);

my @w_param_range=(0.55,0.8,1.0);

#properties
$feature_name = read_feature_name();

sub read_feature_name
{

    open FF, "_FeatureName";
    @content = <FF>;
    close FF;
    return $content[0];
}


sub find_lower
{
    $num = shift;
    for(my $i = 1 ; $i < @order - 1;$i++)
    {
	if($order[$i] == $num)
	{
	    return $order[$i-1];
	}
    }
}

sub find_higher
{
    $num = shift;
    for(my $i = 1 ; $i < @order - 1;$i++)
    {
	if($order[$i] == $num)
	{
	    return $order[$i+1];
	}
    }

}

#$concept = $ARGV[0];
$path = "./results/";
opendir(MODS, $path) || die("Cannot open directory $path");
@files = readdir(MODS);
closedir(MODS);

foreach $bestfile (@files)
{
    if($bestfile =~ /.*\.best$/)
    {
        #print "opening & parsing file $bestfile....\n";
	open( FI, "./results/".$bestfile) || die ("could not open  $bestfile file");
	my @content = ();
	@content = <FI>;
	close FI;
	
	#$count = @content;
	#print("found $count classifiers\n");
	$max = 0.0;
	$parsed_classif=0;
	$cnt = 0;
	$max_index=-1;
	
	foreach $cl (@content)
	{
	    if($cl =~ /^([0-9\.]*)\ ([0-9\.]*)\ ([0-9]*)\ -1\ p\ ([0-9\.]*).*$/)
	    {
		if($4 > $max)	
		{
		    $max=$4;
		    $max_index=$cnt;
		    #print "found next maxima.... $cnt\n";
		}
		$parsed_classif++;
	    }
	    $cnt++;
	}
	if($cnt != $parsed_classif)
	{
	    print "Error: $bestfile, input format wrong";
	}
	if($max_index == -1)
	{
	    printf "$bestfile: no good classifier => adding to shrink\n";
	    push(@shrinktasks, $bestfile);    
	}else
	{
	    $max_line = $content[$max_index];
	    $info_line = "Error";
	    if($max_line =~ /^([0-9\.]*)\ ([0-9\.]*)\ ([0-9]*)\ -1\ p\ ([0-9\.]*).*$/)
	    {
		$info_line = "$bestfile $1 $2 $3 $4";
		#fill out @tasks
		#g-1 c-1
		foreach $w_ (@w_param_range)
		{
		    $g = find_lower($1);
		    $c = find_lower($2);
		    push(@tasks, "$bestfile $g $c $w_");
		}
		#g-1 c+1
		foreach $w_ (@w_param_range)
		{
		    $g = find_lower($1);
		    $c = find_higher($2);
		    push(@tasks, "$bestfile $g $c $w_");
		}
		#g+1 c-1
		foreach $w_ (@w_param_range)
		{
		    $g = find_higher($1);
		    $c = find_lower($2);
		    push(@tasks, "$bestfile $g $c $w_");
		}
		#g+1 c+1
		foreach $w_ (@w_param_range)
		{
		    $g = find_higher($1);
		    $c = find_higher($2);
		    push(@tasks, "$bestfile $g $c $w_");
		}
		
		
	    }
	    print "$bestfile: $info_line\n";
	    #adding infos to tasks array
	        
	}
	
	#exit;
    }
}


    print "==================\nprinting out no. tasks: ";
    $num = @tasks;
    print "$num\n";
    
    print "==================\nprinting out no. shrink tasks: ";
    $num = @shrinktasks;
    print "$num\n";

#####################################################################
#now create sge runner file for training with desired tasks
$task_fname = "sge_train2.pl";
$svm_train_image="./libsvm-/svm-train";
$curr_path = $ENV{PWD};

open FWT, ">$task_fname";
#write head
print FWT "#!/usr/bin/perl -w\nuse lib \"$curr_path\";\nuse MultiTask;\
\@tasks = ();\
\
# sge setup\
my \$dir = \"$curr_path\";\
my \$name = \"TRECtr2\";\
mtSetSgeDir(\$dir);\
mtSetSgeName(\$name);\
mtSetSgeScriptName(\$0);\
\$mem = \"400M\";\
\
mtSetSgeArgs(\"-l ram_free=\$mem\");\
\
# set up environment for mono\
\$ENV{LD_LIBRARY_PATH}=\"/usr/local/share/Mono-2.4/lib\";\
\$ENV{PATH}=\$ENV{PATH} . \":/usr/local/share/Mono/bin\";\
\
chdir \$dir;\
";

#write tasks
print FWT "#TASKS\n";
$tasks_added=0;
foreach $item (@tasks)
{
    if($item =~ /^([0-9]*)\.best\ ([0-9\.]*)\ ([0-9\.]*)\ ([0-9\.]*).*$/)
    {
	$tasks_added++;
	$concept = $1;
	$g = $2;
	$c = $3;
	$w = $4;
	$negw = 1.0 - $w;
	$helpw = 100*$w;
	$feature_name_ex = "${feature_name}.tr";
	$outfile_identif = "${concept}-${feature_name_ex}_${g}_${c}_${helpw}";
	$class_id = "<unknown>";
	print FWT "  push(\@tasks,\"ulimit -t 10800; sleep 10; $svm_train_image -g $g -c $c -w0 $negw -w${concept} $w -m 300 -b 1 \\\"./input/$concept-${feature_name_ex}\\\" \\\"./mod.2/$concept/${outfile_identif}.mod\\\" 	>./logs.2/$concept/${outfile_identif}.sout 2>./logs/$concept/${outfile_identif}.serr\");\n";
    }
}


#write tail
print FWT "\n#run all\n\nMultiTaskGo(\$ARGV[0], \@tasks);\n";

close FWT;
system ("chmod 774 $task_fname");


